%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: 
%           Fusion algorithms developed in [1]. 
% 
% Interface:
%           I_Fus = fusion_algorithms(I_MS, I_PAN, ratio, sensor, imtag, selected_algorithm)
% 
% Inputs:
%           I_MS:               MS image upsampled at PAN scale;
%           I_PAN:              PAN image;
%           ratio:              scaling ratio;
%           sensor:             sensor name;
%           imtag:              tag;
%           selected_algorithm: code of the selected algorithm for fusion.
%
% Outputs:
%           I_Fus:              Fused image.
% 
% Reference:
%       [1] G. Vivone and J. Chanussot, "Fusion of short-wave infrared and visible near-infrared WorldView-3 data", Information Fusion, vol. 61, pp. 71-83, 2020.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function I_Fus = fusion_algorithms(I_MS, I_PAN, ratio, sensor, imtag, selected_algorithm)
    switch selected_algorithm
        case 1
            I_Fus = zeros(size(I_MS));
            if size(I_PAN,3) == 1
                I_Fus = GS(I_MS,I_PAN);
            else
                for ii = 1 : size(I_PAN,3)
                    I_h = GS(I_MS,I_PAN(:,:,ii));
                    I_Fus(:,:,ii) = I_h(:,:,ii);
                end
            end
        case 2
            I_Fus = zeros(size(I_MS));
            if size(I_PAN,3) == 1
                I_Fus = GSA2(I_MS,I_PAN,ratio);
            else
                for ii = 1 : size(I_PAN,3)
                    I_h = GSA2(I_MS,I_PAN(:,:,ii),ratio);
                    I_Fus(:,:,ii) = I_h(:,:,ii);
                end
            end
        case 3
            I_Fus = fusion_GLP_HPM(I_MS, I_PAN, ratio, sensor, imtag, 'Selva');
        case 4
            I_Fus = fusion_GLP_HPF(I_MS, I_PAN, ratio, sensor, imtag, 'Selva');
        case 5
            I_Fus = fusion_GLP_CBD(I_MS, I_PAN, ratio, sensor, imtag, 'Selva');
        case 6
            I_Fus = fusion_GLP_HPM(I_MS, I_PAN, ratio, sensor, imtag, '');
        case 7
            I_Fus = fusion_GLP_HPF(I_MS, I_PAN, ratio, sensor, imtag, '');
        case 8
            I_Fus = fusion_GLP_CBD(I_MS, I_PAN, ratio, sensor, imtag, '');
        otherwise
            I_Fus = zeros(size(I_MS));
    end
end