%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main to test the approaches developed in [1].
% 
% For more information and to have a look at the relative references, please refer to the paper:
% [1] G. Vivone and J. Chanussot, "Fusion of short-wave infrared and visible near-infrared WorldView-3 data", Information Fusion, vol. 61, pp. 71-83, 2020.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
close all;
clear all;

load('Dataset_WV3_Sim_APEX.mat');

flagSpectrometerHS = 3; % Options: 1. S2; 2. S1; 3. S1+S2
flagHypersharpening = 1; % Options: 1. Hypersharpening, 2. Selected bands
selected_algorithm = 5; % Options: 1. GS; 2. GSA; 3. Gauss-HPM; 4. Gauss-HPF; 5. Gauss-CBD; 6. Bil-HPM; 7. Bil-HPF; 8. Bil-CBD 
selected_algorithm_VNIR = 5; % Options: 1. GS; 2. GSA; 3. Gauss-HPM; 4. Gauss-HPF; 5. Gauss-CBD; 6. Bil-HPM; 7. Bil-HPF; 8. Bil-CBD 
selected_algorithm_PAN = 5; % Options: 1. GS; 2. GSA; 3. Gauss-HPM; 4. Gauss-HPF; 5. Gauss-CBD; 6. Bil-HPM; 7. Bil-HPF; 8. Bil-CBD 

%%%%%% Upsampled SWIR to MS size
I_SWIR_MS = imresize(I_SWIR_LR,ratio_SWIR_MS,'bilinear');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fusion Scheme: 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Fusion PAN - MS
I_Fus_PAN_MS = fusion_algorithms(I_MS, I_PAN, ratio_MS_PAN, sensor, imtag, selected_algorithm_VNIR);

%%%%%% Fusion PAN - SWIR
I_Fus_PAN_SWIR = fusion_algorithms(I_SWIR, I_PAN, ratio_MS_PAN*ratio_SWIR_MS, sensor, imtag, selected_algorithm_PAN);

I_Fus_Final_1 = cat(3, I_Fus_PAN_MS, I_Fus_PAN_SWIR);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fusion Scheme: 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Fusion MS - SWIR
if flagHypersharpening == 1
    I_PAN_MS = PAN_HyperSharpening(I_Fus_PAN_MS, I_SWIR, ratio_SWIR_MS*ratio_MS_PAN, flagSpectrometerHS);
else
    I_PAN_MS = PAN_Select(I_Fus_PAN_MS, I_SWIR, ratio_SWIR_MS*ratio_MS_PAN, sensor, imtag);
end

I_Fus_PAN_SWIR = fusion_algorithms(I_SWIR, I_PAN_MS, ratio_SWIR_MS*ratio_MS_PAN, sensor, imtag, selected_algorithm);

I_Fus_Final_2 = cat(3, I_Fus_PAN_MS, I_Fus_PAN_SWIR);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fusion Scheme: 3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if flagHypersharpening == 1
    I_PAN_MS = PAN_HyperSharpening(I_MS_LR, I_SWIR_MS, ratio_SWIR_MS, flagSpectrometerHS);
else
    I_PAN_MS = PAN_Select(I_MS_LR, I_SWIR_MS, ratio_SWIR_MS, sensor, imtag);
end

I_Fus_MS_SWIR = fusion_algorithms(I_SWIR_MS, I_PAN_MS, ratio_SWIR_MS, sensor, imtag, selected_algorithm);

%%%%%% Fusion PAN - (MS - SWIR)
I_Fus_MS_SWIR = interp23tap(I_Fus_MS_SWIR,ratio_MS_PAN);

I_Fus_Pan_SWIR = fusion_algorithms(I_Fus_MS_SWIR, I_PAN, ratio_MS_PAN, sensor, imtag, selected_algorithm_PAN);

I_Fus_Final_3 = cat(3, I_Fus_PAN_MS, I_Fus_Pan_SWIR);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fusion Scheme: 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if flagHypersharpening == 1
    I_PAN_MS = PAN_HyperSharpening(I_Fus_PAN_MS, I_Fus_MS_SWIR, ratio_MS_PAN, flagSpectrometerHS);
else
    I_PAN_MS = PAN_Select(I_Fus_PAN_MS, I_Fus_MS_SWIR, ratio_MS_PAN, sensor, imtag);
end

I_Fus_Pan_MS_SWIR = fusion_algorithms(I_Fus_MS_SWIR, I_PAN_MS, ratio_MS_PAN, sensor, imtag, selected_algorithm);

I_Fus_Final_4 = cat(3, I_Fus_PAN_MS, I_Fus_Pan_MS_SWIR);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%