%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: 
%           Maximum correlation for selecting the HR band as in [1]. 
% 
% Reference:
%       [1] G. Vivone and J. Chanussot, "Fusion of short-wave infrared and visible near-infrared WorldView-3 data", Information Fusion, vol. 61, pp. 71-83, 2020.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [P, r, index_band_max] = maxCorrelationHRImage(I_MS_LR, I_SWIR_band, ratio_SWIR_MS, sensor, imtag, flagfilter)
    
    I_MS_LP = zeros(size(I_MS_LR));
    for ii = 1 : size(I_MS_LP,3)
        I_MS_LP(:,:,ii) = P_LP(I_MS_LR(:,:,ii), sensor, imtag, ratio_SWIR_MS, ii, flagfilter);
    end
    
    r = zeros(1,size(I_MS_LR,3));
    for ii = 1 : size(I_MS_LR,3)
        I_MS_LP_band = I_MS_LP(:,:,ii);
        C = corrcoef(I_SWIR_band(:),I_MS_LP_band(:));
        r(ii) = C(1,2);
    end
    
    [~,index_band_max] = max(r);
    
    P = I_MS_LR(:,:,index_band_max);
    
end